@@ -27,6 +27,12 @@
 
 #include <FLAC/all.h>
 
+#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT < 8
+#define LEGACY_FLAC
+#else
+#undef LEGACY_FLAC
+#endif 
+
 G_BEGIN_DECLS
 
 #define GST_TYPE_FLAC_DEC gst_flac_dec_get_type()
@@ -41,7 +47,11 @@ typedef struct _GstFlacDecClass GstFlacDecClass;
 struct _GstFlacDec {
   GstElement     element;
 
+#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT < 8
   FLAC__SeekableStreamDecoder *seekable_decoder; /* for pull-based operation  */
+#else
+  FLAC__StreamDecoder         *seekable_decoder; /* for pull-based operation  */
+#endif
 
   FLAC__StreamDecoder         *stream_decoder;   /* for chain-based operation */
   GstAdapter                  *adapter;
@@ -51,7 +61,6 @@ struct _GstFlacDec {
   GstPad        *srcpad;
 
   gboolean       init;
-  gboolean       need_newsegment;
 
   guint64        offset;      /* current byte offset of input */
 
@@ -61,6 +70,11 @@ struct _GstFlacDec {
 
   GstSegment     segment;     /* the currently configured segment, in
                                * samples/audio frames (DEFAULT format) */
+  gboolean       running;
+  gboolean       discont;
+  GstEvent      *close_segment;
+  GstEvent      *start_segment;
+  GstTagList    *tags;
 
   GstFlowReturn  last_flow;   /* the last flow return received from either
                                * gst_pad_push or gst_pad_buffer_alloc */
@@ -73,6 +87,8 @@ struct _GstFlacDec {
   /* from the stream info, needed for scanning */
   guint16        min_blocksize;
   guint16        max_blocksize;
+
+  gint64         cur_granulepos; /* only used in framed mode (flac-in-ogg) */
 };
 
 struct _GstFlacDecClass {
