@@ -17,6 +17,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:element-jpegdec
+ *
+ * Decodes jpeg images. If there is no framerate set on sink caps, it sends EOS
+ * after the first picture.
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -192,7 +198,7 @@ gst_jpeg_dec_class_init (GstJpegDecClass * klass)
   GST_DEBUG_CATEGORY_INIT (jpeg_dec_debug, "jpegdec", 0, "JPEG decoder");
 }
 
-static gboolean
+static boolean
 gst_jpeg_dec_fill_input_buffer (j_decompress_ptr cinfo)
 {
 /*
@@ -230,7 +236,7 @@ gst_jpeg_dec_skip_input_data (j_decompress_ptr cinfo, glong num_bytes)
   }
 }
 
-static gboolean
+static boolean
 gst_jpeg_dec_resync_to_restart (j_decompress_ptr cinfo, gint desired)
 {
   GST_DEBUG ("resync_to_start");
@@ -987,9 +993,8 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
             GST_TIME_ARGS (clip_stop - clip_start));
         GST_BUFFER_DURATION (outbuf) = clip_stop - clip_start;
       }
-    } else {
-      GST_WARNING_OBJECT (dec, "Outgoing buffer is outside configured segment");
-    }
+    } else
+      goto drop_buffer;
   }
 
   GST_LOG_OBJECT (dec, "pushing buffer (ts=%" GST_TIME_FORMAT ", dur=%"
@@ -1055,6 +1060,13 @@ alloc_failed:
     }
     goto exit;
   }
+drop_buffer:
+  {
+    GST_WARNING_OBJECT (dec, "Outgoing buffer is outside configured segment");
+    gst_buffer_unref (outbuf);
+    ret = GST_FLOW_OK;
+    goto exit;
+  }
 }
 
 static gboolean
