@@ -1,5 +1,6 @@
 /* GStreamer
  * (c) 2005 Ronald S. Bultje <rbultje@ronald.bitfreak.net>
+ * (c) 2006 Jan Schmidt <thaytan@noraisin.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -46,54 +47,116 @@
 #include "gstautoaudiosink.h"
 #include "gstautodetect.h"
 
+/* Properties */
+enum
+{
+  PROP_0,
+  PROP_CAPS,
+};
+
 static GstStateChangeReturn
 gst_auto_audio_sink_change_state (GstElement * element,
     GstStateChange transition);
+static void gst_auto_audio_sink_dispose (GstAutoAudioSink * sink);
+static void gst_auto_audio_sink_clear_kid (GstAutoAudioSink * sink);
+static void gst_auto_audio_sink_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec);
+static void gst_auto_audio_sink_get_property (GObject * object, guint prop_id,
+    GValue * value, GParamSpec * pspec);
 
 GST_BOILERPLATE (GstAutoAudioSink, gst_auto_audio_sink, GstBin, GST_TYPE_BIN);
 
+static const GstElementDetails gst_auto_audio_sink_details =
+GST_ELEMENT_DETAILS ("Auto audio sink",
+    "Sink/Audio",
+    "Wrapper audio sink for automatically detected audio sink",
+    "Ronald Bultje <rbultje@ronald.bitfreak.net>\n"
+    "Jan Schmidt <thaytan@noraisin.net");
+
+static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_PAD_SINK,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS_ANY);
+
 static void
 gst_auto_audio_sink_base_init (gpointer klass)
 {
   GstElementClass *eklass = GST_ELEMENT_CLASS (klass);
-  const GstElementDetails gst_auto_audio_sink_details =
-      GST_ELEMENT_DETAILS ("Auto audio sink",
-      "Sink/Audio",
-      "Wrapper audio sink for automatically detected audio sink",
-      "Ronald Bultje <rbultje@ronald.bitfreak.net>");
-  GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
-      GST_PAD_SINK,
-      GST_PAD_ALWAYS,
-      GST_STATIC_CAPS_ANY);
 
   gst_element_class_add_pad_template (eklass,
       gst_static_pad_template_get (&sink_template));
+
   gst_element_class_set_details (eklass, &gst_auto_audio_sink_details);
 }
 
 static void
 gst_auto_audio_sink_class_init (GstAutoAudioSinkClass * klass)
 {
-  GstElementClass *eklass = GST_ELEMENT_CLASS (klass);
+  GObjectClass *gobject_class;
+  GstElementClass *eklass;
 
+  gobject_class = G_OBJECT_CLASS (klass);
+  eklass = GST_ELEMENT_CLASS (klass);
+
+  gobject_class->dispose =
+      (GObjectFinalizeFunc) GST_DEBUG_FUNCPTR (gst_auto_audio_sink_dispose);
   eklass->change_state = GST_DEBUG_FUNCPTR (gst_auto_audio_sink_change_state);
+  gobject_class->set_property =
+      GST_DEBUG_FUNCPTR (gst_auto_audio_sink_set_property);
+  gobject_class->get_property =
+      GST_DEBUG_FUNCPTR (gst_auto_audio_sink_get_property);
+
+  /**
+   * GstAutoAudioSink:filter-caps
+   *
+   * This property will filter out candidate sinks that can handle the specified
+   * caps. By default only audio sinks that support raw floating point and
+   * integer audio are selected.
+   *
+   * This property can only be set before the element goes to the READY state.
+   *
+   * Since: 0.10.7
+   **/
+  g_object_class_install_property (gobject_class, PROP_CAPS,
+      g_param_spec_boxed ("filter-caps", "Filter caps",
+          "Filter sink candidates using these caps.", GST_TYPE_CAPS,
+          G_PARAM_READWRITE));
+}
+
+static void
+gst_auto_audio_sink_dispose (GstAutoAudioSink * sink)
+{
+  gst_auto_audio_sink_clear_kid (sink);
+
+  if (sink->filter_caps)
+    gst_caps_unref (sink->filter_caps);
+  sink->filter_caps = NULL;
+
+  G_OBJECT_CLASS (parent_class)->dispose ((GObject *) sink);
+}
+
+static void
+gst_auto_audio_sink_clear_kid (GstAutoAudioSink * sink)
+{
+  if (sink->kid) {
+    gst_element_set_state (sink->kid, GST_STATE_NULL);
+    gst_bin_remove (GST_BIN (sink), sink->kid);
+    sink->kid = NULL;
+  }
 }
 
 /*
  * Hack to make initial linking work; ideally, this'd work even when
  * no target has been assigned to the ghostpad yet.
  */
-
 static void
 gst_auto_audio_sink_reset (GstAutoAudioSink * sink)
 {
   GstPad *targetpad;
 
+  gst_auto_audio_sink_clear_kid (sink);
+
   /* fakesink placeholder */
-  if (sink->kid) {
-    gst_element_set_state (sink->kid, GST_STATE_NULL);
-    gst_bin_remove (GST_BIN (sink), sink->kid);
-  }
   sink->kid = gst_element_factory_make ("fakesink", "tempsink");
   gst_bin_add (GST_BIN (sink), sink->kid);
 
@@ -103,6 +166,9 @@ gst_auto_audio_sink_reset (GstAutoAudioSink * sink)
   gst_object_unref (targetpad);
 }
 
+static GstStaticCaps raw_caps =
+    GST_STATIC_CAPS ("audio/x-raw-int; audio/x-raw-float");
+
 static void
 gst_auto_audio_sink_init (GstAutoAudioSink * sink,
     GstAutoAudioSinkClass * g_class)
@@ -112,6 +178,10 @@ gst_auto_audio_sink_init (GstAutoAudioSink * sink,
 
   gst_auto_audio_sink_reset (sink);
 
+  /* set the default raw audio caps */
+  sink->filter_caps = gst_static_caps_get (&raw_caps);
+
+  /* mark as sink */
   GST_OBJECT_FLAG_SET (sink, GST_ELEMENT_IS_SINK);
 }
 
@@ -179,6 +249,9 @@ gst_auto_audio_sink_find_best (GstAutoAudioSink * sink)
   GstMessage *message = NULL;
   GSList *errors = NULL;
   GstBus *bus = gst_bus_new ();
+  GstPad *el_pad = NULL;
+  GstCaps *el_caps = NULL, *intersect = NULL;
+  gboolean no_match = TRUE;
 
   list = gst_registry_feature_filter (gst_registry_get_default (),
       (GstPluginFeatureFilter) gst_auto_audio_sink_factory_filter, FALSE, sink);
@@ -197,23 +270,42 @@ gst_auto_audio_sink_find_best (GstAutoAudioSink * sink)
       GstStateChangeReturn ret;
 
       GST_DEBUG_OBJECT (sink, "Testing %s", GST_PLUGIN_FEATURE (f)->name);
+
+      /* If autoaudiosink has been provided with filter caps,
+       * accept only sinks that match with the filter caps */
+      if (sink->filter_caps) {
+        el_pad = gst_element_get_static_pad (GST_ELEMENT (el), "sink");
+        el_caps = gst_pad_get_caps (el_pad);
+        gst_object_unref (el_pad);
+        GST_DEBUG_OBJECT (sink,
+            "Checking caps: %" GST_PTR_FORMAT " vs. %" GST_PTR_FORMAT,
+            sink->filter_caps, el_caps);
+        intersect = gst_caps_intersect (sink->filter_caps, el_caps);
+        no_match = gst_caps_is_empty (intersect);
+        gst_caps_unref (el_caps);
+        gst_caps_unref (intersect);
+
+        if (no_match) {
+          GST_DEBUG_OBJECT (sink, "Incompatible caps");
+          gst_object_unref (el);
+          continue;
+        } else {
+          GST_DEBUG_OBJECT (sink, "Found compatible caps");
+        }
+      }
+
       gst_element_set_bus (el, bus);
       ret = gst_element_set_state (el, GST_STATE_READY);
       if (ret == GST_STATE_CHANGE_SUCCESS) {
-        gst_element_set_state (el, GST_STATE_NULL);
         GST_DEBUG_OBJECT (sink, "This worked!");
         choice = el;
         break;
       }
 
       /* collect all error messages */
-      while ((message = gst_bus_pop (bus))) {
-        if (GST_MESSAGE_TYPE (message) == GST_MESSAGE_ERROR) {
-          GST_DEBUG_OBJECT (sink, "error message %" GST_PTR_FORMAT, message);
-          errors = g_slist_append (errors, message);
-        } else {
-          gst_message_unref (message);
-        }
+      while ((message = gst_bus_pop_filtered (bus, GST_MESSAGE_ERROR))) {
+        GST_DEBUG_OBJECT (sink, "error message %" GST_PTR_FORMAT, message);
+        errors = g_slist_append (errors, message);
       }
 
       gst_element_set_state (el, GST_STATE_NULL);
@@ -230,9 +322,13 @@ gst_auto_audio_sink_find_best (GstAutoAudioSink * sink)
       GST_DEBUG_OBJECT (sink, "reposting message %p", errors->data);
       gst_element_post_message (GST_ELEMENT (sink), GST_MESSAGE (errors->data));
     } else {
-      /* general fallback */
-      GST_ELEMENT_ERROR (sink, LIBRARY, INIT, (NULL),
-          ("Failed to find a supported audio sink"));
+      /* send warning message to application and use a fakesink */
+      GST_ELEMENT_WARNING (sink, RESOURCE, NOT_FOUND, (NULL),
+          ("Failed to find a usable audio sink"));
+      choice = gst_element_factory_make ("fakesink", "fake-audio-sink");
+      if (g_object_class_find_property (G_OBJECT_GET_CLASS (choice), "sync"))
+        g_object_set (choice, "sync", TRUE, NULL);
+      gst_element_set_state (choice, GST_STATE_READY);
     }
   }
   gst_object_unref (bus);
@@ -249,11 +345,7 @@ gst_auto_audio_sink_detect (GstAutoAudioSink * sink)
   GstElement *esink;
   GstPad *targetpad;
 
-  if (sink->kid) {
-    gst_element_set_state (sink->kid, GST_STATE_NULL);
-    gst_bin_remove (GST_BIN (sink), sink->kid);
-    sink->kid = NULL;
-  }
+  gst_auto_audio_sink_clear_kid (sink);
 
   /* find element */
   GST_DEBUG_OBJECT (sink, "Creating new kid");
@@ -262,7 +354,12 @@ gst_auto_audio_sink_detect (GstAutoAudioSink * sink)
   }
 
   sink->kid = esink;
-  gst_element_set_state (sink->kid, GST_STATE (sink));
+  /* Ensure the child is brought up to the right state to match the parent
+   * although it's currently always in READY and 
+   * we're always doing NULL->READY. */
+  if (GST_STATE (sink->kid) < GST_STATE (sink))
+    gst_element_set_state (sink->kid, GST_STATE (sink));
+
   gst_bin_add (GST_BIN (sink), esink);
 
   /* attach ghost pad */
@@ -305,3 +402,38 @@ gst_auto_audio_sink_change_state (GstElement * element,
 
   return ret;
 }
+
+static void
+gst_auto_audio_sink_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec)
+{
+  GstAutoAudioSink *sink = GST_AUTO_AUDIO_SINK (object);
+
+  switch (prop_id) {
+    case PROP_CAPS:
+      if (sink->filter_caps)
+        gst_caps_unref (sink->filter_caps);
+      sink->filter_caps = gst_caps_copy (gst_value_get_caps (value));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
+static void
+gst_auto_audio_sink_get_property (GObject * object, guint prop_id,
+    GValue * value, GParamSpec * pspec)
+{
+  GstAutoAudioSink *sink = GST_AUTO_AUDIO_SINK (object);
+
+  switch (prop_id) {
+    case PROP_CAPS:{
+      gst_value_set_caps (value, sink->filter_caps);
+      break;
+    }
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
