@@ -153,6 +153,9 @@ static GstStaticPadTemplate video_sink_factory =
         "systemstream = (boolean) FALSE; "
         "video/x-huffyuv, "
         "width = (int) [ 16, 4096 ], "
+        "height = (int) [ 16, 4096 ], " "framerate = (fraction) [ 0, MAX ];"
+        "video/x-dirac, "
+        "width = (int) [ 16, 4096 ], "
         "height = (int) [ 16, 4096 ], " "framerate = (fraction) [ 0, MAX ]")
     );
 
@@ -552,6 +555,8 @@ gst_avi_mux_vidsink_set_caps (GstPad * pad, GstCaps * vscaps)
           avipad->vids.compression = GST_MAKE_FOURCC ('M', 'P', 'E', 'G');
           break;
       }
+    } else if (!strcmp (mimetype, "video/x-dirac")) {
+      avipad->vids.compression = GST_MAKE_FOURCC ('d', 'r', 'a', 'c');
     }
 
     if (!avipad->vids.compression)
