@@ -37,13 +37,16 @@
 GST_DEBUG_CATEGORY_EXTERN (id3demux_debug);
 #define GST_CAT_DEFAULT (id3demux_debug)
 
-static gchar *parse_comment_frame (ID3TagsWorking * work);
+static gboolean parse_comment_frame (ID3TagsWorking * work);
+static gchar *parse_url_link_frame (ID3TagsWorking * work,
+    const gchar ** tag_name);
 static GArray *parse_text_identification_frame (ID3TagsWorking * work);
 static gchar *parse_user_text_identification_frame (ID3TagsWorking * work,
     const gchar ** tag_name);
 static gchar *parse_unique_file_identifier (ID3TagsWorking * work,
     const gchar ** tag_name);
 static gboolean parse_relative_volume_adjustment_two (ID3TagsWorking * work);
+static void parse_obsolete_tdat_frame (ID3TagsWorking * work);
 static gboolean id3v2_tag_to_taglist (ID3TagsWorking * work,
     const gchar * tag_name, const gchar * tag_str);
 /* Parse a single string into an array of gchar* */
@@ -60,8 +63,6 @@ static gboolean parse_picture_frame (ID3TagsWorking * work);
 #define ID3V2_ENCODING_UTF16BE 0x02
 #define ID3V2_ENCODING_UTF8    0x03
 
-extern guint read_synch_uint (guint8 * data, guint size);
-
 gboolean
 id3demux_id3v2_parse_frame (ID3TagsWorking * work)
 {
@@ -73,6 +74,10 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
   gchar *tag_str = NULL;
   GArray *tag_fields = NULL;
 
+#ifdef HAVE_ZLIB
+  guint8 *uncompressed_data = NULL;
+#endif
+
   /* Check that the frame id is valid */
   for (i = 0; i < 5 && work->frame_id[i] != '\0'; i++) {
     if (!g_ascii_isalnum (work->frame_id[i])) {
@@ -96,6 +101,7 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
   if (tag_name == NULL &&
       strncmp (work->frame_id, "RVA2", 4) != 0 &&
       strncmp (work->frame_id, "TXXX", 4) != 0 &&
+      strncmp (work->frame_id, "TDAT", 4) != 0 &&
       strncmp (work->frame_id, "UFID", 4) != 0) {
     return FALSE;
   }
@@ -121,22 +127,23 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
     uLongf destSize = work->parse_size;
     Bytef *dest, *src;
 
-    work->parse_data = g_malloc (work->parse_size);
-    g_return_val_if_fail (work->parse_data != NULL, FALSE);
+    uncompressed_data = g_malloc (work->parse_size);
 
-    dest = (Bytef *) work->parse_data;
+    dest = (Bytef *) uncompressed_data;
     src = (Bytef *) frame_data;
 
     if (uncompress (dest, &destSize, src, frame_data_size) != Z_OK) {
-      g_free (work->parse_data);
+      g_free (uncompressed_data);
       return FALSE;
     }
     if (destSize != work->parse_size) {
       GST_WARNING
-          ("Decompressing ID3v2 frame %s did not produce expected size %d bytes (got %d)",
-          tag_name, work->parse_data, destSize);
+          ("Decompressing ID3v2 frame %s did not produce expected size %d bytes (got %lu)",
+          tag_name, work->parse_size, destSize);
+      g_free (uncompressed_data);
       return FALSE;
     }
+    work->parse_data = uncompressed_data;
 #else
     GST_WARNING ("Compressed ID3v2 tag frame could not be decompressed"
         " because gstid3demux was compiled without zlib support");
@@ -147,16 +154,22 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
   }
 
   if (work->frame_id[0] == 'T') {
-    if (strcmp (work->frame_id, "TXXX") != 0) {
-      /* Text identification frame */
-      tag_fields = parse_text_identification_frame (work);
-    } else {
+    if (strcmp (work->frame_id, "TDAT") == 0) {
+      parse_obsolete_tdat_frame (work);
+      result = TRUE;
+    } else if (strcmp (work->frame_id, "TXXX") == 0) {
       /* Handle user text frame */
       tag_str = parse_user_text_identification_frame (work, &tag_name);
+    } else {
+      /* Text identification frame */
+      tag_fields = parse_text_identification_frame (work);
     }
+  } else if (work->frame_id[0] == 'W' && strcmp (work->frame_id, "WXXX") != 0) {
+    /* URL link frame: ISO-8859-1 encoded, one frame per tag */
+    tag_str = parse_url_link_frame (work, &tag_name);
   } else if (!strcmp (work->frame_id, "COMM")) {
     /* Comment */
-    tag_str = parse_comment_frame (work);
+    result = parse_comment_frame (work);
   } else if (!strcmp (work->frame_id, "APIC")) {
     /* Attached picture */
     result = parse_picture_frame (work);
@@ -167,9 +180,13 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
     /* Unique file identifier */
     tag_str = parse_unique_file_identifier (work, &tag_name);
   }
-
-  if (work->frame_flags & ID3V2_FRAME_FORMAT_COMPRESSION)
-    g_free (work->parse_data);
+#ifdef HAVE_ZLIB
+  if (work->frame_flags & ID3V2_FRAME_FORMAT_COMPRESSION) {
+    g_free (uncompressed_data);
+    uncompressed_data = NULL;
+    work->parse_data = frame_data;
+  }
+#endif
 
   if (tag_str != NULL) {
     /* g_print ("Tag %s value %s\n", tag_name, tag_str); */
@@ -195,23 +212,23 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
   return result;
 }
 
-static gchar *
+static gboolean
 parse_comment_frame (ID3TagsWorking * work)
 {
+  guint dummy;
   guint8 encoding;
   gchar language[4];
   GArray *fields = NULL;
-  gchar *out_str = NULL;
   gchar *description, *text;
 
   if (work->parse_size < 6)
-    return NULL;
+    return FALSE;
 
   encoding = work->parse_data[0];
-  language[0] = work->parse_data[1];
-  language[1] = work->parse_data[2];
-  language[2] = work->parse_data[3];
-  language[3] = 0;
+  language[0] = g_ascii_tolower (work->parse_data[1]);
+  language[1] = g_ascii_tolower (work->parse_data[2]);
+  language[2] = g_ascii_tolower (work->parse_data[3]);
+  language[3] = '\0';
 
   parse_split_strings (encoding, (gchar *) work->parse_data + 4,
       work->parse_size - 4, &fields);
@@ -226,19 +243,41 @@ parse_comment_frame (ID3TagsWorking * work)
   if (!g_utf8_validate (text, -1, NULL)) {
     GST_WARNING ("Converted string is not valid utf-8");
     goto fail;
-  } else {
-    if (strlen (description) > 0 && g_utf8_validate (description, -1, NULL)) {
-      out_str = g_strdup_printf ("Description: %s\nComment: %s",
-          description, text);
-    } else {
-      out_str = g_strdup (text);
-    }
   }
 
-fail:
-  free_tag_strings (fields);
+  /* skip our own dummy descriptions (from id3v2mux) */
+  if (strlen (description) > 0 && g_utf8_validate (description, -1, NULL) &&
+      sscanf (description, "c%u", &dummy) != 1) {
+    gchar *s;
 
-  return out_str;
+    /* must be either an ISO-639-1 or ISO-639-2 language code */
+    if (language[0] != '\0' &&
+        g_ascii_isalpha (language[0]) &&
+        g_ascii_isalpha (language[1]) &&
+        (g_ascii_isalpha (language[2]) || language[2] == '\0')) {
+      s = g_strdup_printf ("%s[%s]=%s", description, language, text);
+    } else {
+      s = g_strdup_printf ("%s=%s", description, text);
+    }
+    gst_tag_list_add (work->tags, GST_TAG_MERGE_APPEND,
+        GST_TAG_EXTENDED_COMMENT, s, NULL);
+    g_free (s);
+  } else if (text != NULL && *text != '\0') {
+    gst_tag_list_add (work->tags, GST_TAG_MERGE_APPEND,
+        GST_TAG_COMMENT, text, NULL);
+  } else {
+    goto fail;
+  }
+
+  free_tag_strings (fields);
+  return TRUE;
+
+fail:
+  {
+    GST_WARNING ("failed to parse COMM frame");
+    free_tag_strings (fields);
+    return FALSE;
+  }
 }
 
 static GArray *
@@ -255,18 +294,75 @@ parse_text_identification_frame (ID3TagsWorking * work)
       work->parse_size - 1, &fields);
   if (fields) {
     if (fields->len > 0) {
-      GST_LOG ("Read %d fields from Text ID frame of size %d. First is '%s'",
-          fields->len, work->parse_size - 1,
+      GST_LOG ("Read %d fields from Text ID frame of size %d with encoding %d"
+          ". First is '%s'", fields->len, work->parse_size - 1, encoding,
           g_array_index (fields, gchar *, 0));
     } else {
-      GST_LOG ("Read %d fields from Text ID frame of size %d", fields->len,
-          work->parse_size - 1);
+      GST_LOG ("Read 0 fields from Text ID frame of size %d with encoding %d",
+          work->parse_size - 1, encoding);
     }
   }
 
   return fields;
 }
 
+static gboolean
+link_is_known_license (const gchar * url)
+{
+  return g_str_has_prefix (url, "http://creativecommons.org/licenses/");
+}
+
+static gchar *
+parse_url_link_frame (ID3TagsWorking * work, const gchar ** tag_name)
+{
+  gsize len;
+  gchar *nul, *data, *link;
+
+  *tag_name = NULL;
+
+  if (work->parse_size == 0)
+    return NULL;
+
+  data = (gchar *) work->parse_data;
+  /* if there's more data then the string is long, we only want to parse the
+   * data up to the terminating zero to g_convert and ignore the rest, as
+   * per spec */
+  nul = memchr (data, '\0', work->parse_size);
+  if (nul != NULL) {
+    len = (gsize) (nul - data);
+  } else {
+    len = work->parse_size;
+  }
+
+  link = g_convert (data, len, "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
+
+  if (link == NULL || !gst_uri_is_valid (link)) {
+    GST_DEBUG ("Invalid URI in %s frame: %s", work->frame_id,
+        GST_STR_NULL (link));
+    g_free (link);
+    return NULL;
+  }
+
+  /* we don't know if it's a link to a page that explains the copyright
+   * situation, or a link that points to/represents a license, the ID3 spec
+   * does not separate those two things; for now only put known license URIs
+   * into GST_TAG_LICENSE_URI and everything else into GST_TAG_COPYRIGHT_URI */
+  if (strcmp (work->frame_id, "WCOP") == 0) {
+    if (link_is_known_license (link))
+      *tag_name = GST_TAG_LICENSE_URI;
+    else
+      *tag_name = GST_TAG_COPYRIGHT_URI;
+  } else if (strcmp (work->frame_id, "WOAF") == 0) {
+    /* can't be bothered to create a CONTACT_URI tag for this, so let's just
+     * put into into GST_TAG_CONTACT, which is where it ends up when reading
+     * the info from vorbis comments as well */
+    *tag_name = GST_TAG_CONTACT;
+  }
+
+  return link;
+}
+
+
 static gchar *
 parse_user_text_identification_frame (ID3TagsWorking * work,
     const gchar ** tag_name)
@@ -401,10 +497,10 @@ parse_picture_frame (ID3TagsWorking * work)
   GstCaps *image_caps = NULL;
   gboolean is_pic_uri = FALSE;
   guint8 txt_encoding, pic_type;
-  gchar *mime_str = NULL;
+  gchar *c, *mime_str = NULL;
   gint len, datalen;
 
-  GST_LOG ("APIC frame");
+  GST_LOG ("APIC frame (ID3v2.%u)", ID3V2_VER_MAJOR (work->hdr.version));
 
   if (work->parse_size < 1 + 1 + 1 + 1 + 1)
     goto not_enough_data;
@@ -413,8 +509,32 @@ parse_picture_frame (ID3TagsWorking * work)
   ++work->parse_data;
   --work->parse_size;
 
-  if (!parse_id_string (work, &mime_str, &len, &datalen)) {
-    return FALSE;
+  /* Read image format; in early ID3v2 versions this is a fixed-length
+   * 3-character string without terminator; in later versions (>= 2.3.0)
+   * this is a NUL-terminated string of variable length */
+  if (ID3V2_VER_MAJOR (work->hdr.version) < 3) {
+    if (work->parse_size < 3)
+      goto not_enough_data;
+
+    mime_str = g_strndup ((gchar *) work->parse_data, 3);
+    len = 3;
+  } else {
+    if (!parse_id_string (work, &mime_str, &len, &datalen))
+      return FALSE;
+    ++len;                      /* for string terminator */
+  }
+
+  /* Fix up 'jpg' => 'jpeg' in mime/media type */
+  if (mime_str != NULL &&
+      (g_ascii_strcasecmp (mime_str, "jpg") == 0 ||
+          g_ascii_strcasecmp (mime_str, "image/jpg") == 0)) {
+    g_free (mime_str);
+    mime_str = g_strdup ("jpeg");
+  }
+
+  /* Make lower-case */
+  for (c = mime_str; c != NULL && *c != '\0'; ++c) {
+    *c = g_ascii_tolower (*c);
   }
 
   is_pic_uri = (mime_str != NULL && strcmp (mime_str, "-->") == 0);
@@ -427,12 +547,13 @@ parse_picture_frame (ID3TagsWorking * work)
     mime_str = tmp;
   }
 
-  if (work->parse_size < (len + 1) + 1 + 1 + 1)
+  if (work->parse_size < len + 1 + 1 + 1)
     goto not_enough_data;
 
-  work->parse_data += (len + 1);
-  work->parse_size -= (len + 1);
+  work->parse_data += len;
+  work->parse_size -= len;
 
+  /* Read image type */
   pic_type = work->parse_data[0];
   ++work->parse_data;
   --work->parse_size;
@@ -495,25 +616,29 @@ parse_picture_frame (ID3TagsWorking * work)
   }
 
   if (image && image_caps) {
-    /* FIXME: remove #ifdef once we depend on -base >= 0.10.9 */
-#ifdef GST_TYPE_TAG_IMAGE_TYPE
-    if (pic_type > 0x14)
-      pic_type = GST_TAG_IMAGE_TYPE_UNDEFINED;
-    gst_structure_set (gst_caps_get_structure (image_caps, 0),
-        "image-type", GST_TYPE_TAG_IMAGE_TYPE,
-        (GstTagImageType) pic_type, NULL);
-#endif
 
-    gst_buffer_set_caps (image, image_caps);
-    gst_caps_unref (image_caps);
     if (pic_type == 0x01 || pic_type == 0x02) {
-      /* file icon of some sort */
+      /* file icon for preview. Don't add image-type to caps, since there
+       * is only supposed to be one of these. */
+      gst_buffer_set_caps (image, image_caps);
       gst_tag_list_add (work->tags, GST_TAG_MERGE_APPEND,
           GST_TAG_PREVIEW_IMAGE, image, NULL);
     } else {
+      GstTagImageType gst_tag_pic_type = GST_TAG_IMAGE_TYPE_UNDEFINED;
+
+      /* Remap the ID3v2 APIC type our ImageType enum */
+      if (pic_type >= 0x3 && pic_type <= 0x14)
+        gst_tag_pic_type = (GstTagImageType) (pic_type - 2);
+
+      gst_structure_set (gst_caps_get_structure (image_caps, 0),
+          "image-type", GST_TYPE_TAG_IMAGE_TYPE, gst_tag_pic_type, NULL);
+
+      gst_buffer_set_caps (image, image_caps);
       gst_tag_list_add (work->tags, GST_TAG_MERGE_APPEND,
           GST_TAG_IMAGE, image, NULL);
     }
+
+    gst_caps_unref (image_caps);
     gst_buffer_unref (image);
   }
 
@@ -611,6 +736,23 @@ parse_relative_volume_adjustment_two (ID3TagsWorking * work)
   return (gain_tag_name != NULL || peak_tag_name != NULL);
 }
 
+static void
+parse_obsolete_tdat_frame (ID3TagsWorking * work)
+{
+  if (work->parse_size >= 5 &&
+      work->parse_data[0] == ID3V2_ENCODING_ISO8859 &&
+      g_ascii_isdigit (work->parse_data[1]) &&
+      g_ascii_isdigit (work->parse_data[2]) &&
+      g_ascii_isdigit (work->parse_data[3]) &&
+      g_ascii_isdigit (work->parse_data[4])) {
+    work->pending_day = (10 * g_ascii_digit_value (work->parse_data[1])) +
+        g_ascii_digit_value (work->parse_data[2]);
+    work->pending_month = (10 * g_ascii_digit_value (work->parse_data[3])) +
+        g_ascii_digit_value (work->parse_data[4]);
+    GST_LOG ("date (dd/mm) %02u/%02u", work->pending_day, work->pending_month);
+  }
+}
+
 static gboolean
 id3v2_tag_to_taglist (ID3TagsWorking * work, const gchar * tag_name,
     const gchar * tag_str)
@@ -668,14 +810,23 @@ id3v2_tag_to_taglist (ID3TagsWorking * work, const gchar * tag_name,
       break;
     }
     case G_TYPE_STRING:{
-      if (!strcmp (tag_name, GST_TAG_GENRE)) {
-        if (work->prev_genre && !strcmp (tag_str, work->prev_genre))
-          break;                /* Same as the last genre */
-        g_free (work->prev_genre);
-        work->prev_genre = g_strdup (tag_str);
+      const GValue *val;
+      guint i, num;
+
+      /* make sure we add each unique string only once per tag, we don't want
+       * to have the same genre in the genre list multiple times, for example,
+       * or the same DiscID in there twice just because it's contained in the
+       * tag multiple times under different TXXX user tags */
+      num = gst_tag_list_get_tag_size (tag_list, tag_name);
+      for (i = 0; i < num; ++i) {
+        val = gst_tag_list_get_value_index (tag_list, tag_name, i);
+        if (val != NULL && strcmp (g_value_get_string (val), tag_str) == 0)
+          break;
+      }
+      if (i == num) {
+        gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
+            tag_name, tag_str, NULL);
       }
-      gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
-          tag_name, tag_str, NULL);
       break;
     }
 
@@ -893,8 +1044,11 @@ parse_insert_string_field (guint8 encoding, gchar * data, gint data_size,
 
       break;
     case ID3V2_ENCODING_ISO8859:
-      field = g_convert (data, data_size, "UTF-8", "ISO-8859-1",
-          NULL, NULL, NULL);
+      if (g_utf8_validate (data, data_size, NULL))
+        field = g_strndup (data, data_size);
+      else
+        field = g_convert (data, data_size, "UTF-8", "ISO-8859-1",
+            NULL, NULL, NULL);
       break;
     default:
       field = g_strndup (data, data_size);
