@@ -161,6 +161,7 @@
 #define GST_MATROSKA_CODEC_ID_VIDEO_REALVIDEO3   "V_REAL/RV30"
 #define GST_MATROSKA_CODEC_ID_VIDEO_REALVIDEO4   "V_REAL/RV40"
 #define GST_MATROSKA_CODEC_ID_VIDEO_THEORA       "V_THEORA"
+#define GST_MATROSKA_CODEC_ID_VIDEO_DIRAC        "V_DIRAC"
 /* TODO: Quicktime */
 
 #define GST_MATROSKA_CODEC_ID_AUDIO_MPEG1_L1     "A_MPEG/L1"
@@ -254,6 +255,7 @@ typedef struct _GstMatroskaTrackContext {
   GstPad       *pad;
   GstCaps      *caps;
   guint         index;
+  GstFlowReturn last_flow;
 
   /* some often-used info */
   gchar        *codec_id, *codec_name, *name, *language;
