@@ -79,8 +79,8 @@ GST_BOILERPLATE (GstRtpTheoraPay, gst_rtp_theora_pay, GstBaseRTPPayload,
 
 static gboolean gst_rtp_theora_pay_setcaps (GstBaseRTPPayload * basepayload,
     GstCaps * caps);
-static GstStateChangeReturn gst_rtp_theora_pay_change_state (GstElement * element,
-    GstStateChange transition);
+static GstStateChangeReturn gst_rtp_theora_pay_change_state (GstElement *
+    element, GstStateChange transition);
 static GstFlowReturn gst_rtp_theora_pay_handle_buffer (GstBaseRTPPayload * pad,
     GstBuffer * buffer);
 
@@ -127,7 +127,7 @@ gst_rtp_theora_pay_init (GstRtpTheoraPay * rtptheorapay,
 }
 
 static void
-gst_rtp_theora_pay_cleanup (GstRtpTheoraPay *rtptheorapay)
+gst_rtp_theora_pay_cleanup (GstRtpTheoraPay * rtptheorapay)
 {
   g_list_foreach (rtptheorapay->headers, (GFunc) gst_mini_object_unref, NULL);
   g_list_free (rtptheorapay->headers);
@@ -675,7 +675,8 @@ header_error:
 }
 
 static GstStateChangeReturn
-gst_rtp_theora_pay_change_state (GstElement * element, GstStateChange transition)
+gst_rtp_theora_pay_change_state (GstElement * element,
+    GstStateChange transition)
 {
   GstRtpTheoraPay *rtptheorapay;
   GstStateChangeReturn ret;
