@@ -1,6 +1,9 @@
 /*
  * GStreamer - SunAudio mixer interface element
- * Copyright (C) 2005,2006 Brian Cameron <brian.cameron@sun.com>
+ * Copyright (C) 2005,2006,2008 Sun Microsystems, Inc.,
+ *               Brian Cameron <brian.cameron@sun.com>
+ * Copyright (C) 2008 Sun Microsystems, Inc.,
+ *               Jan Schmidt <jan.schmidt@sun.com> 
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -39,8 +42,6 @@
 GST_DEBUG_CATEGORY_EXTERN (sunaudio_debug);
 #define GST_CAT_DEFAULT sunaudio_debug
 
-#define SCALE_FACTOR 2.55       /* 255/100 */
-
 static gboolean
 gst_sunaudiomixer_ctrl_open (GstSunAudioMixerCtrl * mixer)
 {
@@ -187,16 +188,15 @@ gst_sunaudiomixer_ctrl_get_volume (GstSunAudioMixerCtrl * mixer,
 
   switch (sunaudiotrack->track_num) {
     case GST_SUNAUDIO_TRACK_OUTPUT:
-      gain = (int) ((float) audioinfo.play.gain / (float) SCALE_FACTOR + 0.5);
+      gain = (int) audioinfo.play.gain;
       balance = audioinfo.play.balance;
       break;
     case GST_SUNAUDIO_TRACK_LINE_IN:
-      gain = (int) ((float) audioinfo.record.gain / (float) SCALE_FACTOR + 0.5);
+      gain = (int) audioinfo.record.gain;
       balance = audioinfo.record.balance;
       break;
     case GST_SUNAUDIO_TRACK_MONITOR:
-      gain =
-          (int) ((float) audioinfo.monitor_gain / (float) SCALE_FACTOR + 0.5);
+      gain = (int) audioinfo.monitor_gain;
       balance = audioinfo.record.balance;
       break;
   }
@@ -272,15 +272,15 @@ gst_sunaudiomixer_ctrl_set_volume (GstSunAudioMixerCtrl * mixer,
   r_real_gain = volumes[1];
 
   if (l_real_gain == r_real_gain) {
-    gain = (int) ((float) l_real_gain * (float) SCALE_FACTOR + 0.5);
+    gain = l_real_gain;
     balance = AUDIO_MID_BALANCE;
   } else if (l_real_gain < r_real_gain) {
-    gain = (int) ((float) r_real_gain * (float) SCALE_FACTOR + 0.5);
+    gain = r_real_gain;
     ratio = (float) l_real_gain / (float) r_real_gain;
     balance =
         AUDIO_RIGHT_BALANCE - (int) (ratio * (float) AUDIO_MID_BALANCE + 0.5);
   } else {
-    gain = (int) ((float) l_real_gain * (float) SCALE_FACTOR + 0.5);
+    gain = l_real_gain;
     ratio = (float) r_real_gain / (float) l_real_gain;
     balance =
         AUDIO_LEFT_BALANCE + (int) (ratio * (float) AUDIO_MID_BALANCE + 0.5);
