@@ -26,7 +26,7 @@
  *
  * <refsect2>
  * v4l2src can be used to capture video from v4l2 devices, like webcams and tv cards.
- * <title>Example launch line</title>
+ * <title>Example launch lines</title>
  * <para>
  * <programlisting>
  * gst-launch v4l2src ! xvimagesink
@@ -34,6 +34,13 @@
  * This pipeline shows the video captured from /dev/video0 tv card and for
  * webcams.
  * </para>
+ * <para>
+ * <programlisting>
+ * gst-launch-0.10 v4l2src ! jpegdec ! xvimagesink
+ * </programlisting>
+ * This pipeline shows the video captured from a webcam that delivers jpeg
+ * images.
+ * </para>
  * </refsect2>
  */
 
@@ -58,14 +65,17 @@ GST_ELEMENT_DETAILS ("Video (video4linux2/raw) Source",
     "Ronald Bultje <rbultje@ronald.bitfreak.net>,"
     " Edgard Lima <edgard.lima@indt.org.br>");
 
-
 GST_DEBUG_CATEGORY (v4l2src_debug);
 #define GST_CAT_DEFAULT v4l2src_debug
 
+#define DEFAULT_PROP_ALWAYS_COPY        TRUE
+
 enum
 {
   PROP_0,
   V4L2_STD_OBJECT_PROPS,
+  PROP_QUEUE_SIZE,
+  PROP_ALWAYS_COPY
 };
 
 static const guint32 gst_v4l2_formats[] = {
@@ -223,15 +233,17 @@ GST_BOILERPLATE_FULL (GstV4l2Src, gst_v4l2src, GstPushSrc, GST_TYPE_PUSH_SRC,
     gst_v4l2src_init_interfaces);
 
 static void gst_v4l2src_dispose (GObject * object);
+static void gst_v4l2src_finalize (GstV4l2Src * v4l2src);
 
 /* basesrc methods */
 static gboolean gst_v4l2src_start (GstBaseSrc * src);
 static gboolean gst_v4l2src_stop (GstBaseSrc * src);
 static gboolean gst_v4l2src_set_caps (GstBaseSrc * src, GstCaps * caps);
 static GstCaps *gst_v4l2src_get_caps (GstBaseSrc * src);
+static gboolean gst_v4l2src_query (GstBaseSrc * bsrc, GstQuery * query);
 static GstFlowReturn gst_v4l2src_create (GstPushSrc * src, GstBuffer ** out);
 
-static void gst_v4l2src_fixate (GstPad * pad, GstCaps * caps);
+static void gst_v4l2src_fixate (GstBaseSrc * basesrc, GstCaps * caps);
 
 static void gst_v4l2src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -270,40 +282,52 @@ gst_v4l2src_class_init (GstV4l2SrcClass * klass)
   pushsrc_class = GST_PUSH_SRC_CLASS (klass);
 
   gobject_class->dispose = gst_v4l2src_dispose;
+  gobject_class->finalize = (GObjectFinalizeFunc) gst_v4l2src_finalize;
   gobject_class->set_property = gst_v4l2src_set_property;
   gobject_class->get_property = gst_v4l2src_get_property;
 
   gst_v4l2_object_install_properties_helper (gobject_class);
+  g_object_class_install_property (gobject_class, PROP_QUEUE_SIZE,
+      g_param_spec_uint ("queue-size", "Queue size",
+          "Number of buffers to be enqueud in the driver",
+          GST_V4L2_MIN_BUFFERS, GST_V4L2_MAX_BUFFERS, GST_V4L2_MIN_BUFFERS,
+          G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_ALWAYS_COPY,
+      g_param_spec_boolean ("always-copy", "Always Copy",
+          "If the buffer will or not be used directly from mmap",
+          DEFAULT_PROP_ALWAYS_COPY, G_PARAM_READWRITE));
 
-  basesrc_class->get_caps = gst_v4l2src_get_caps;
-  basesrc_class->set_caps = gst_v4l2src_set_caps;
-  basesrc_class->start = gst_v4l2src_start;
-  basesrc_class->stop = gst_v4l2src_stop;
+  basesrc_class->get_caps = GST_DEBUG_FUNCPTR (gst_v4l2src_get_caps);
+  basesrc_class->set_caps = GST_DEBUG_FUNCPTR (gst_v4l2src_set_caps);
+  basesrc_class->start = GST_DEBUG_FUNCPTR (gst_v4l2src_start);
+  basesrc_class->stop = GST_DEBUG_FUNCPTR (gst_v4l2src_stop);
+  basesrc_class->query = GST_DEBUG_FUNCPTR (gst_v4l2src_query);
+  basesrc_class->fixate = GST_DEBUG_FUNCPTR (gst_v4l2src_fixate);
 
-  pushsrc_class->create = gst_v4l2src_create;
+  pushsrc_class->create = GST_DEBUG_FUNCPTR (gst_v4l2src_create);
 }
 
 static void
 gst_v4l2src_init (GstV4l2Src * v4l2src, GstV4l2SrcClass * klass)
 {
+  /* fixme: give an update_fps_function */
   v4l2src->v4l2object = gst_v4l2_object_new (GST_ELEMENT (v4l2src),
-      gst_v4l2_get_input, gst_v4l2_set_input, gst_v4l2src_update_fps);
+      gst_v4l2_get_input, gst_v4l2_set_input, NULL);
 
-  v4l2src->breq.count = 0;
+  /* number of buffers requested */
+  v4l2src->num_buffers = GST_V4L2_MIN_BUFFERS;
+
+  v4l2src->always_copy = DEFAULT_PROP_ALWAYS_COPY;
 
   v4l2src->formats = NULL;
 
-  /* fps */
-  v4l2src->fps_n = 0;
-  v4l2src->fps_d = 1;
-
   v4l2src->is_capturing = FALSE;
 
-  gst_pad_set_fixatecaps_function (GST_BASE_SRC_PAD (v4l2src),
-      gst_v4l2src_fixate);
-
   gst_base_src_set_format (GST_BASE_SRC (v4l2src), GST_FORMAT_TIME);
   gst_base_src_set_live (GST_BASE_SRC (v4l2src), TRUE);
+
+  v4l2src->fps_d = 0;
+  v4l2src->fps_n = 0;
 }
 
 
@@ -320,18 +344,30 @@ gst_v4l2src_dispose (GObject * object)
 }
 
 
+static void
+gst_v4l2src_finalize (GstV4l2Src * v4l2src)
+{
+  gst_v4l2_object_destroy (v4l2src->v4l2object);
+
+  G_OBJECT_CLASS (parent_class)->finalize ((GObject *) (v4l2src));
+}
+
+
 static void
 gst_v4l2src_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec)
 {
-  GstV4l2Src *v4l2src;
-
-  g_return_if_fail (GST_IS_V4L2SRC (object));
-  v4l2src = GST_V4L2SRC (object);
+  GstV4l2Src *v4l2src = GST_V4L2SRC (object);
 
   if (!gst_v4l2_object_set_property_helper (v4l2src->v4l2object,
           prop_id, value, pspec)) {
     switch (prop_id) {
+      case PROP_QUEUE_SIZE:
+        v4l2src->num_buffers = g_value_get_uint (value);
+        break;
+      case PROP_ALWAYS_COPY:
+        v4l2src->always_copy = g_value_get_boolean (value);
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -344,14 +380,17 @@ static void
 gst_v4l2src_get_property (GObject * object,
     guint prop_id, GValue * value, GParamSpec * pspec)
 {
-  GstV4l2Src *v4l2src;
-
-  g_return_if_fail (GST_IS_V4L2SRC (object));
-  v4l2src = GST_V4L2SRC (object);
+  GstV4l2Src *v4l2src = GST_V4L2SRC (object);
 
   if (!gst_v4l2_object_get_property_helper (v4l2src->v4l2object,
           prop_id, value, pspec)) {
     switch (prop_id) {
+      case PROP_QUEUE_SIZE:
+        g_value_set_uint (value, v4l2src->num_buffers);
+        break;
+      case PROP_ALWAYS_COPY:
+        g_value_set_boolean (value, v4l2src->always_copy);
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -362,15 +401,12 @@ gst_v4l2src_get_property (GObject * object,
 
 /* this function is a bit of a last resort */
 static void
-gst_v4l2src_fixate (GstPad * pad, GstCaps * caps)
+gst_v4l2src_fixate (GstBaseSrc * basesrc, GstCaps * caps)
 {
   GstStructure *structure;
   gint i;
-  G_GNUC_UNUSED gchar *caps_str;
 
-  caps_str = gst_caps_to_string (caps);
-  GST_DEBUG_OBJECT (GST_PAD_PARENT (pad), "fixating caps %s", caps_str);
-  g_free (caps_str);
+  GST_DEBUG_OBJECT (basesrc, "fixating caps %" GST_PTR_FORMAT, caps);
 
   for (i = 0; i < gst_caps_get_size (caps); ++i) {
     structure = gst_caps_get_structure (caps, i);
@@ -379,7 +415,7 @@ gst_v4l2src_fixate (GstPad * pad, GstCaps * caps)
     /* FIXME such sizes? we usually fixate to something in the 320x200
      * range... */
     /* We are fixating to greater possble size (limited to GST_V4L2_MAX_SIZE)
-       and framarate closer to 15/2 that is common in web-cams */
+       and framerate closer to 15/2 that is common in web-cams */
     gst_structure_fixate_field_nearest_int (structure, "width",
         GST_V4L2_MAX_SIZE);
     gst_structure_fixate_field_nearest_int (structure, "height",
@@ -399,10 +435,19 @@ gst_v4l2src_fixate (GstPad * pad, GstCaps * caps)
 }
 
 static GstStructure *
-gst_v4l2src_v4l2fourcc_to_caps (guint32 fourcc)
+gst_v4l2src_v4l2fourcc_to_structure (guint32 fourcc)
 {
   GstStructure *structure = NULL;
 
+  /* FIXME: new FourCCs
+     camera: ZC0301 PC Camera
+     driver: zc0301
+     BA81, S910, PWC1, PWC2
+
+     camera:
+     driver:
+   */
+
   switch (fourcc) {
     case V4L2_PIX_FMT_MJPEG:   /* Motion-JPEG */
     case V4L2_PIX_FMT_JPEG:    /* JFIF JPEG */
@@ -553,6 +598,9 @@ gst_v4l2src_v4l2fourcc_to_caps (guint32 fourcc)
       break;
     case V4L2_PIX_FMT_WNVA:    /* Winnov hw compres */
       break;
+    case V4L2_PIX_FMT_SBGGR8:
+      structure = gst_structure_new ("video/x-raw-bayer", NULL);
+      break;
     default:
       GST_DEBUG ("Unknown fourcc 0x%08x %" GST_FOURCC_FORMAT,
           fourcc, GST_FOURCC_ARGS (fourcc));
@@ -562,11 +610,146 @@ gst_v4l2src_v4l2fourcc_to_caps (guint32 fourcc)
   return structure;
 }
 
-static guint32
-gst_v4l2_fourcc_from_structure (GstStructure * structure)
+static struct v4l2_fmtdesc *
+gst_v4l2src_get_format_from_fourcc (GstV4l2Src * v4l2src, guint32 fourcc)
 {
-  guint32 fourcc = 0;
-  const gchar *mimetype = gst_structure_get_name (structure);
+  struct v4l2_fmtdesc *fmt;
+  GSList *walk;
+
+  if (fourcc == 0)
+    return NULL;
+
+  walk = v4l2src->formats;
+  while (walk) {
+    fmt = (struct v4l2_fmtdesc *) walk->data;
+    if (fmt->pixelformat == fourcc)
+      return fmt;
+    /* special case for jpeg */
+    if ((fmt->pixelformat == V4L2_PIX_FMT_MJPEG && fourcc == V4L2_PIX_FMT_JPEG)
+        || (fmt->pixelformat == V4L2_PIX_FMT_JPEG
+            && fourcc == V4L2_PIX_FMT_MJPEG)) {
+      return fmt;
+    }
+    walk = g_slist_next (walk);
+  }
+
+  return NULL;
+}
+
+static GstCaps *
+gst_v4l2src_get_all_caps (void)
+{
+  static GstCaps *caps = NULL;
+
+  if (caps == NULL) {
+    GstStructure *structure;
+    guint i;
+
+    caps = gst_caps_new_empty ();
+    for (i = 0; i < GST_V4L2_FORMAT_COUNT; i++) {
+      structure = gst_v4l2src_v4l2fourcc_to_structure (gst_v4l2_formats[i]);
+      if (structure) {
+        gst_structure_set (structure,
+            "width", GST_TYPE_INT_RANGE, 1, GST_V4L2_MAX_SIZE,
+            "height", GST_TYPE_INT_RANGE, 1, GST_V4L2_MAX_SIZE,
+            "framerate", GST_TYPE_FRACTION_RANGE, 0, 1, 100, 1, NULL);
+        gst_caps_append_structure (caps, structure);
+      }
+    }
+  }
+
+  return caps;
+}
+
+static GstCaps *
+gst_v4l2src_get_caps (GstBaseSrc * src)
+{
+  GstV4l2Src *v4l2src = GST_V4L2SRC (src);
+  GstCaps *ret;
+  GSList *walk;
+
+  if (!GST_V4L2_IS_OPEN (v4l2src->v4l2object)) {
+    /* FIXME: copy? */
+    return
+        gst_caps_copy (gst_pad_get_pad_template_caps (GST_BASE_SRC_PAD
+            (v4l2src)));
+  }
+
+  if (v4l2src->probed_caps)
+    return gst_caps_ref (v4l2src->probed_caps);
+
+  if (!v4l2src->formats)
+    gst_v4l2src_fill_format_list (v4l2src);
+
+  ret = gst_caps_new_empty ();
+
+  for (walk = v4l2src->formats; walk; walk = walk->next) {
+    struct v4l2_fmtdesc *format;
+    GstStructure *template;
+
+    format = (struct v4l2_fmtdesc *) walk->data;
+
+    template = gst_v4l2src_v4l2fourcc_to_structure (format->pixelformat);
+
+    if (template) {
+      GstCaps *tmp;
+
+      tmp = gst_v4l2src_probe_caps_for_format (v4l2src, format->pixelformat,
+          template);
+      if (tmp)
+        gst_caps_append (ret, tmp);
+
+      gst_structure_free (template);
+    } else {
+      GST_DEBUG_OBJECT (v4l2src, "unknown format %u", format->pixelformat);
+    }
+  }
+
+  v4l2src->probed_caps = gst_caps_ref (ret);
+
+  GST_INFO_OBJECT (v4l2src, "probed caps: %" GST_PTR_FORMAT, ret);
+
+  return ret;
+}
+
+/* collect data for the given caps
+ * @caps: given input caps
+ * @format: location for the v4l format
+ * @w/@h: location for width and height
+ * @fps_n/@fps_d: location for framerate
+ * @size: location for expected size of the frame or 0 if unknown
+ */
+static gboolean
+gst_v4l2_get_caps_info (GstV4l2Src * v4l2src, GstCaps * caps,
+    struct v4l2_fmtdesc **format, gint * w, gint * h, guint * fps_n,
+    guint * fps_d, guint * size)
+{
+  GstStructure *structure;
+  const GValue *framerate;
+  guint32 fourcc;
+  const gchar *mimetype;
+  guint outsize;
+
+  /* default unknown values */
+  fourcc = 0;
+  outsize = 0;
+
+  structure = gst_caps_get_structure (caps, 0);
+
+  if (!gst_structure_get_int (structure, "width", w))
+    return FALSE;
+
+  if (!gst_structure_get_int (structure, "height", h))
+    return FALSE;
+
+  framerate = gst_structure_get_value (structure, "framerate");
+  if (!framerate)
+    return FALSE;
+
+  *fps_n = gst_value_get_fraction_numerator (framerate);
+  *fps_d = gst_value_get_fraction_denominator (framerate);
+
+  mimetype = gst_structure_get_name (structure);
 
   if (!strcmp (mimetype, "video/x-raw-yuv")) {
     gst_structure_get_fourcc (structure, "format", &fourcc);
@@ -575,24 +758,35 @@ gst_v4l2_fourcc_from_structure (GstStructure * structure)
       case GST_MAKE_FOURCC ('I', '4', '2', '0'):
       case GST_MAKE_FOURCC ('I', 'Y', 'U', 'V'):
         fourcc = V4L2_PIX_FMT_YUV420;
+        outsize = GST_ROUND_UP_4 (*w) * GST_ROUND_UP_2 (*h);
+        outsize += 2 * ((GST_ROUND_UP_8 (*w) / 2) * (GST_ROUND_UP_2 (*h) / 2));
         break;
       case GST_MAKE_FOURCC ('Y', 'U', 'Y', '2'):
         fourcc = V4L2_PIX_FMT_YUYV;
+        outsize = (GST_ROUND_UP_2 (*w) * 2) * *h;
         break;
       case GST_MAKE_FOURCC ('Y', '4', '1', 'P'):
         fourcc = V4L2_PIX_FMT_Y41P;
+        outsize = (GST_ROUND_UP_2 (*w) * 2) * *h;
         break;
       case GST_MAKE_FOURCC ('U', 'Y', 'V', 'Y'):
         fourcc = V4L2_PIX_FMT_UYVY;
+        outsize = (GST_ROUND_UP_2 (*w) * 2) * *h;
         break;
       case GST_MAKE_FOURCC ('Y', 'V', '1', '2'):
         fourcc = V4L2_PIX_FMT_YVU420;
+        outsize = GST_ROUND_UP_4 (*w) * GST_ROUND_UP_2 (*h);
+        outsize += 2 * ((GST_ROUND_UP_8 (*w) / 2) * (GST_ROUND_UP_2 (*h) / 2));
         break;
       case GST_MAKE_FOURCC ('Y', '4', '1', 'B'):
         fourcc = V4L2_PIX_FMT_YUV411P;
+        outsize = GST_ROUND_UP_4 (*w) * *h;
+        outsize += 2 * ((GST_ROUND_UP_8 (*w) / 4) * *h);
         break;
       case GST_MAKE_FOURCC ('Y', '4', '2', 'B'):
         fourcc = V4L2_PIX_FMT_YUV422P;
+        outsize = GST_ROUND_UP_4 (*w) * *h;
+        outsize += 2 * ((GST_ROUND_UP_8 (*w) / 2) * *h);
         break;
     }
   } else if (!strcmp (mimetype, "video/x-raw-rgb")) {
@@ -627,135 +821,13 @@ gst_v4l2_fourcc_from_structure (GstStructure * structure)
     fourcc = V4L2_PIX_FMT_JPEG;
   }
 
-  return fourcc;
-}
-
-static struct v4l2_fmtdesc *
-gst_v4l2src_get_format_from_fourcc (GstV4l2Src * v4l2src, guint32 fourcc)
-{
-  struct v4l2_fmtdesc *fmt;
-  GSList *walk;
-
   if (fourcc == 0)
-    return NULL;
+    return FALSE;
 
-  walk = v4l2src->formats;
-  while (walk) {
-    fmt = (struct v4l2_fmtdesc *) walk->data;
-    if (fmt->pixelformat == fourcc)
-      return fmt;
-    /* special case for jpeg */
-    if ((fmt->pixelformat == V4L2_PIX_FMT_MJPEG && fourcc == V4L2_PIX_FMT_JPEG)
-        || (fmt->pixelformat == V4L2_PIX_FMT_JPEG
-            && fourcc == V4L2_PIX_FMT_MJPEG)) {
-      return fmt;
-    }
-    walk = g_slist_next (walk);
-  }
+  *format = gst_v4l2src_get_format_from_fourcc (v4l2src, fourcc);
+  *size = outsize;
 
-  return NULL;
-}
-
-static struct v4l2_fmtdesc *
-gst_v4l2_caps_to_v4l2fourcc (GstV4l2Src * v4l2src, GstStructure * structure)
-{
-  return gst_v4l2src_get_format_from_fourcc (v4l2src,
-      gst_v4l2_fourcc_from_structure (structure));
-}
-
-static GstCaps *
-gst_v4l2src_get_all_caps (void)
-{
-  static GstCaps *caps = NULL;
-
-  if (caps == NULL) {
-    GstStructure *structure;
-    guint i;
-
-    caps = gst_caps_new_empty ();
-    for (i = 0; i < GST_V4L2_FORMAT_COUNT; i++) {
-      structure = gst_v4l2src_v4l2fourcc_to_caps (gst_v4l2_formats[i]);
-      if (structure) {
-        gst_structure_set (structure,
-            "width", GST_TYPE_INT_RANGE, 1, GST_V4L2_MAX_SIZE,
-            "height", GST_TYPE_INT_RANGE, 1, GST_V4L2_MAX_SIZE,
-            "framerate", GST_TYPE_FRACTION_RANGE, 0, 1, 100, 1, NULL);
-        gst_caps_append_structure (caps, structure);
-      }
-    }
-  }
-
-  return caps;
-}
-
-static GstCaps *
-gst_v4l2src_get_caps (GstBaseSrc * src)
-{
-  GstV4l2Src *v4l2src = GST_V4L2SRC (src);
-  GstCaps *caps;
-  struct v4l2_fmtdesc *format;
-  int min_w, max_w, min_h, max_h;
-  GSList *walk;
-  GstStructure *structure;
-  guint fps_n, fps_d;
-
-  if (!GST_V4L2_IS_OPEN (v4l2src->v4l2object)) {
-    return
-        gst_caps_copy (gst_pad_get_pad_template_caps (GST_BASE_SRC_PAD
-            (v4l2src)));
-  }
-
-  if (!v4l2src->formats)
-    gst_v4l2src_fill_format_list (v4l2src);
-
-  /* build our own capslist */
-  caps = gst_caps_new_empty ();
-  walk = v4l2src->formats;
-  if (!gst_v4l2src_get_fps (v4l2src, &fps_n, &fps_d)) {
-    GST_DEBUG_OBJECT (v4l2src, "frame rate is unknown.");
-    fps_n = 0;
-    fps_d = 1;
-  }
-  while (walk) {
-    format = (struct v4l2_fmtdesc *) walk->data;
-    walk = g_slist_next (walk);
-
-    /* get size delimiters */
-    if (!gst_v4l2src_get_size_limits (v4l2src, format,
-            &min_w, &max_w, &min_h, &max_h)) {
-      continue;
-    }
-    /* template, FIXME, why limit if the device reported correct results? */
-    /* we are doing it right now to avoid unexpected results */
-    min_w = CLAMP (min_w, 1, GST_V4L2_MAX_SIZE);
-    min_h = CLAMP (min_h, 1, GST_V4L2_MAX_SIZE);
-    max_w = CLAMP (max_w, min_w, GST_V4L2_MAX_SIZE);
-    max_h = CLAMP (max_h, min_h, GST_V4L2_MAX_SIZE);
-
-    /* add to list */
-    structure = gst_v4l2src_v4l2fourcc_to_caps (format->pixelformat);
-
-    if (structure) {
-      gst_structure_set (structure,
-          "width", GST_TYPE_INT_RANGE, min_w, max_w,
-          "height", GST_TYPE_INT_RANGE, min_h, max_h, NULL);
-
-      /* FIXME, why random range? */
-      /* AFAIK: in v4l2 standard we still don't have a good way to enum
-         for a range. Currently it is on discussion on v4l2 forum.
-         Currently, something more smart could be done for tvcard devices.
-         The maximum value could be determined by 'v4l2_standard.frameperiod'
-         and minimum also to the same if V4L2_CAP_TIMEPERFRAME is not set. */
-      /* another approach for web-cams would be to try to set a very
-         high(100/1) and low(1/1) FPSs and get the values returned */
-      gst_structure_set (structure, "framerate", GST_TYPE_FRACTION_RANGE,
-          0, 1, 100, 1, NULL);
-
-      gst_caps_append_structure (caps, structure);
-    }
-  }
-
-  return caps;
+  return TRUE;
 }
 
 static gboolean
@@ -763,10 +835,9 @@ gst_v4l2src_set_caps (GstBaseSrc * src, GstCaps * caps)
 {
   GstV4l2Src *v4l2src;
   gint w = 0, h = 0;
-  GstStructure *structure;
   struct v4l2_fmtdesc *format;
-  const GValue *framerate;
   guint fps_n, fps_d;
+  guint size;
 
   v4l2src = GST_V4L2SRC (src);
 
@@ -776,70 +847,90 @@ gst_v4l2src_set_caps (GstBaseSrc * src, GstCaps * caps)
 
   /* make sure we stop capturing and dealloc buffers */
   if (GST_V4L2_IS_ACTIVE (v4l2src->v4l2object)) {
+    /* both will throw an element-error on failure */
     if (!gst_v4l2src_capture_stop (v4l2src))
       return FALSE;
     if (!gst_v4l2src_capture_deinit (v4l2src))
       return FALSE;
   }
 
-  structure = gst_caps_get_structure (caps, 0);
-
   /* we want our own v4l2 type of fourcc codes */
-  if (!(format = gst_v4l2_caps_to_v4l2fourcc (v4l2src, structure))) {
+  if (!gst_v4l2_get_caps_info (v4l2src, caps, &format, &w, &h, &fps_n, &fps_d,
+          &size)) {
+    GST_DEBUG_OBJECT (v4l2src,
+        "can't get capture format from caps %" GST_PTR_FORMAT, caps);
     return FALSE;
   }
 
-  gst_structure_get_int (structure, "width", &w);
-  gst_structure_get_int (structure, "height", &h);
-  framerate = gst_structure_get_value (structure, "framerate");
+  GST_DEBUG_OBJECT (v4l2src, "trying to set_capture %dx%d at %d/%d fps, "
+      "format %s", w, h, fps_n, fps_d, format->description);
 
-  GST_DEBUG_OBJECT (v4l2src, "trying to set_capture %dx%d, format %s",
-      w, h, format->description);
-
-  if (framerate) {
-    fps_n = gst_value_get_fraction_numerator (framerate);
-    fps_d = gst_value_get_fraction_denominator (framerate);
-  } else {
-    fps_n = 0;
-    fps_d = 1;
-  }
-
-  if (!gst_v4l2src_set_capture (v4l2src, format, &w, &h, &fps_n, &fps_d)) {
-    GST_WARNING_OBJECT (v4l2src, "could not set_capture %dx%d, format %s",
-        w, h, format->description);
+  if (!gst_v4l2src_set_capture (v4l2src, format->pixelformat, w, h, fps_n,
+          fps_d))
+    /* error already posted */
     return FALSE;
-  }
 
-  if (fps_n) {
-    gst_structure_set (structure,
-        "width", G_TYPE_INT, w,
-        "height", G_TYPE_INT, h,
-        "framerate", GST_TYPE_FRACTION, fps_n, fps_d, NULL);
-  } else {
-    gst_structure_set (structure,
-        "width", G_TYPE_INT, w, "height", G_TYPE_INT, h, NULL);
-  }
-
-  if (!gst_v4l2src_capture_init (v4l2src))
+  if (!gst_v4l2src_capture_init (v4l2src, caps))
     return FALSE;
 
   if (!gst_v4l2src_capture_start (v4l2src))
     return FALSE;
 
-  if (v4l2src->fps_n != fps_n || v4l2src->fps_d != fps_d) {
-    GST_WARNING_OBJECT (v4l2src,
-        "framerate changed after start capturing from %u/%u to %u/%u", fps_n,
-        fps_d, v4l2src->fps_n, v4l2src->fps_d);
-    if (fps_n) {
-      gst_structure_set (structure,
-          "width", G_TYPE_INT, w,
-          "height", G_TYPE_INT, h,
-          "framerate", GST_TYPE_FRACTION, v4l2src->fps_n, v4l2src->fps_d, NULL);
-    }
-  }
+  /* now store the expected output size */
+  v4l2src->frame_byte_size = size;
+
   return TRUE;
 }
 
+static gboolean
+gst_v4l2src_query (GstBaseSrc * bsrc, GstQuery * query)
+{
+  GstV4l2Src *src;
+  gboolean res = FALSE;
+
+  src = GST_V4L2SRC (bsrc);
+
+  switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_LATENCY:{
+      GstClockTime min_latency, max_latency;
+
+      /* device must be open */
+      if (!GST_V4L2_IS_OPEN (src->v4l2object))
+        goto done;
+
+      /* we must have a framerate */
+      if (src->fps_n <= 0 || src->fps_d <= 0)
+        goto done;
+
+      /* min latency is the time to capture one frame */
+      min_latency =
+          gst_util_uint64_scale_int (GST_SECOND, src->fps_d, src->fps_n);
+
+      /* max latency is total duration of the frame buffer */
+      /* FIXME: what to use here? */
+      max_latency = 1 * min_latency;
+
+      GST_DEBUG_OBJECT (bsrc,
+          "report latency min %" GST_TIME_FORMAT " max %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (min_latency), GST_TIME_ARGS (max_latency));
+
+      /* we are always live, the min latency is 1 frame and the max latency is
+       * the complete buffer of frames. */
+      gst_query_set_latency (query, TRUE, min_latency, max_latency);
+
+      res = TRUE;
+      break;
+    }
+    default:
+      res = GST_BASE_SRC_CLASS (parent_class)->query (bsrc, query);
+      break;
+  }
+
+done:
+
+  return res;
+}
+
 /* start and stop are not symmetric -- start will open the device, but not start
  * capture. it's setcaps that will start capture, which is called via basesrc's
  * negotiate method. stop will both stop capture and close the device.
@@ -849,6 +940,7 @@ gst_v4l2src_start (GstBaseSrc * src)
 {
   GstV4l2Src *v4l2src = GST_V4L2SRC (src);
 
+  /* open the device */
   if (!gst_v4l2_object_start (v4l2src->v4l2object))
     return FALSE;
 
@@ -871,9 +963,13 @@ gst_v4l2src_stop (GstBaseSrc * src)
       return FALSE;
   }
 
+  /* close the device */
   if (!gst_v4l2_object_stop (v4l2src->v4l2object))
     return FALSE;
 
+  v4l2src->fps_d = 0;
+  v4l2src->fps_n = 0;
+
   return TRUE;
 }
 
@@ -883,11 +979,11 @@ gst_v4l2src_get_read (GstV4l2Src * v4l2src, GstBuffer ** buf)
   gint amount;
   gint buffersize;
 
-  buffersize = v4l2src->format.fmt.pix.sizeimage;
+  buffersize = v4l2src->frame_byte_size;
+
+  *buf = gst_buffer_new_and_alloc (buffersize);
 
   do {
-    *buf = gst_v4l2src_buffer_new (v4l2src, buffersize, NULL, NULL);
-
     amount =
         read (v4l2src->v4l2object->video_fd, GST_BUFFER_DATA (*buf),
         buffersize);
@@ -898,10 +994,51 @@ gst_v4l2src_get_read (GstV4l2Src * v4l2src, GstBuffer ** buf)
         continue;
       } else
         goto read_error;
-    } else
-      goto short_read;
+    } else {
+      /* short reads can happen if a signal interrupts the read */
+      continue;
+    }
   } while (TRUE);
 
+  GST_BUFFER_OFFSET (*buf) = v4l2src->offset++;
+  GST_BUFFER_OFFSET_END (*buf) = v4l2src->offset;
+  /* timestamps, LOCK to get clock and base time. */
+  {
+    GstClock *clock;
+    GstClockTime timestamp;
+
+    GST_OBJECT_LOCK (v4l2src);
+    if ((clock = GST_ELEMENT_CLOCK (v4l2src))) {
+      /* we have a clock, get base time and ref clock */
+      timestamp = GST_ELEMENT (v4l2src)->base_time;
+      gst_object_ref (clock);
+    } else {
+      /* no clock, can't set timestamps */
+      timestamp = GST_CLOCK_TIME_NONE;
+    }
+    GST_OBJECT_UNLOCK (v4l2src);
+
+    if (clock) {
+      GstClockTime latency;
+
+      /* the time now is the time of the clock minus the base time */
+      timestamp = gst_clock_get_time (clock) - timestamp;
+      gst_object_unref (clock);
+
+      latency =
+          gst_util_uint64_scale_int (GST_SECOND, v4l2src->fps_d,
+          v4l2src->fps_n);
+
+      if (timestamp > latency)
+        timestamp -= latency;
+      else
+        timestamp = 0;
+    }
+
+    /* FIXME: use the timestamp from the buffer itself! */
+    GST_BUFFER_TIMESTAMP (*buf) = timestamp;
+  }
+
   return GST_FLOW_OK;
 
   /* ERRORS */
@@ -913,63 +1050,49 @@ read_error:
     gst_buffer_unref (*buf);
     return GST_FLOW_ERROR;
   }
-short_read:
-  {
-    GST_ELEMENT_ERROR (v4l2src, RESOURCE, SYNC,
-        (_("Error reading from device '%s'"),
-            v4l2src->v4l2object->videodev),
-        ("Error read()ing a buffer on device %s: got only %d bytes instead of expected %d.",
-            v4l2src->v4l2object->videodev, amount, buffersize));
-    gst_buffer_unref (*buf);
-    return GST_FLOW_ERROR;
-  }
 }
 
 static GstFlowReturn
 gst_v4l2src_get_mmap (GstV4l2Src * v4l2src, GstBuffer ** buf)
 {
-  gint i, num;
+  GstBuffer *temp;
+  GstFlowReturn ret;
+  guint size;
+  guint count;
 
-  /* grab a frame from the device, post an error */
-  num = gst_v4l2src_grab_frame (v4l2src);
-  if (num == -1)
-    goto grab_failed;
+  count = 0;
 
-  i = v4l2src->format.fmt.pix.sizeimage;
+again:
+  ret = gst_v4l2src_grab_frame (v4l2src, &temp);
+  if (ret != GST_FLOW_OK)
+    goto done;
 
-  /* check if this is the last buffer in the queue. If so do a memcpy to put it back asap
-     to avoid framedrops and deadlocks because of stupid elements */
-  if (g_atomic_int_get (&v4l2src->pool->refcount) == v4l2src->breq.count) {
-    GST_LOG_OBJECT (v4l2src, "using memcpy'd buffer");
+  if (v4l2src->frame_byte_size > 0) {
+    size = GST_BUFFER_SIZE (temp);
 
-    *buf = gst_v4l2src_buffer_new (v4l2src, i, NULL, NULL);
-    memcpy (GST_BUFFER_DATA (*buf), v4l2src->pool->buffers[num].start, i);
+    /* if size does not match what we expected, try again */
+    if (size != v4l2src->frame_byte_size) {
+      GST_ELEMENT_WARNING (v4l2src, RESOURCE, READ,
+          (_("Got unexpected frame size of %u instead of %u."),
+              size, v4l2src->frame_byte_size), (NULL));
+      gst_buffer_unref (temp);
+      if (count++ > 50)
+        goto size_error;
 
-    /* posts an error message if something went wrong */
-    if (!gst_v4l2src_queue_frame (v4l2src, num))
-      goto queue_failed;
-  } else {
-    GST_LOG_OBJECT (v4l2src, "using mmap'd buffer");
-    *buf =
-        gst_v4l2src_buffer_new (v4l2src, i, v4l2src->pool->buffers[num].start,
-        &v4l2src->pool->buffers[num]);
-
-    /* no need to be careful here, both are > 0, because the element uses them */
-    g_atomic_int_inc (&v4l2src->pool->buffers[num].refcount);
-    g_atomic_int_inc (&v4l2src->pool->refcount);
+      goto again;
+    }
   }
-  return GST_FLOW_OK;
+
+  *buf = temp;
+done:
+  return ret;
 
   /* ERRORS */
-grab_failed:
+size_error:
   {
-    GST_DEBUG_OBJECT (v4l2src, "failed to grab a frame");
-    return GST_FLOW_ERROR;
-  }
-queue_failed:
-  {
-    GST_DEBUG_OBJECT (v4l2src, "failed to queue frame");
-    gst_buffer_unref (*buf);
+    GST_ELEMENT_ERROR (v4l2src, RESOURCE, READ,
+        (_("Error read()ing %d bytes on device '%s'."),
+            v4l2src->frame_byte_size, v4l2src->v4l2object->videodev), (NULL));
     return GST_FLOW_ERROR;
   }
 }
@@ -980,7 +1103,7 @@ gst_v4l2src_create (GstPushSrc * src, GstBuffer ** buf)
   GstV4l2Src *v4l2src = GST_V4L2SRC (src);
   GstFlowReturn ret;
 
-  if (v4l2src->breq.memory == V4L2_MEMORY_MMAP) {
+  if (v4l2src->use_mmap) {
     ret = gst_v4l2src_get_mmap (v4l2src, buf);
   } else {
     ret = gst_v4l2src_get_read (v4l2src, buf);
